<?php
/**
 * MVCLight
 *
 * An open source application development framework for PHP
 *
 * @package		MVCLight
 * @author		Jorge Albaladejo Pomares
 * @copyright	Copyright (c) 2011, Jorge Albaladejo Pomares
 * @license		http://jorgealbaladejo.com/portfolio/MVCLight/license.txt
 * @link		http://jorgealbaladejo.com/MVCLight
 * @since		Version 0.1a
 * @filesource
 */

/**
 * system/lib/language.class.php
 * 
 * Manages the entire localization and internationalization process
 * 
 */
class Language
{
	/**
     * Singleton object reference
     * 
     */
    private static $singleton = null;
	
	/**
	 * language code
	 *
	 */
	private $langCode = null;
	
	/**
	 * Registry reference
	 *  
	 */
	private $registry = null;
	
	/**
	 * Flag for gettext available 
	 * 
	 */
	private $gettext = null; 
	
	/**
	 * Keeps the translation strings rom the correspondent file in memory
	 * 
	 */
	private $langBuffer = null;
	
	/**
	 * Constructor
	 *
	 */
	function __construct()
	{
		$this->registry = Registry::getInstance();
		
		$this->path		= $this->registry->get('lang_path');
		$this->domain	= $this->registry->get('lang_domain');
		$this->charset	= $this->registry->get('charset');
		$this->langCode = $this->registry->get('lang_code');		
	}
	
	/**
	 * The main destructor erases singleton object
	 * 
	 * @return void
	 */
	public function __destruct() 
	{  			
      	self::$singleton = null ;
  	}
	
	/**
	 * __clone private so nobody can clone the instance
	 *
	 * @access private
	 */
	private function __clone(){}
	
	/**
     * Retrieves the default router instance.
     *
     * @return Router
     * 
     */
    public static function getInstance()
    {
        if (self::$singleton === null) 
        {
            self::$singleton = new Language();
        }

        return self::$singleton;
    }
	
	/**
	 * Initializes gettext and locale settings
	 * 
	 * @param string $lang the locale code to be set
	 * @param string $domain the domain to bind translations to
	 *
	 * @return boolean the result of the operation
	 *
	 * @access public
	 * 
	 * @TODO: could not make gettext work on my server... further research needed
	 */
    public function initGetText($lang = '')
    {
		// variables used
		$localeSet 			= '';
		$bindtextdomainSet 	= '';
		$path				= $this->path;
		$domain				= $this->domain;
		$charset			= $this->charset;
		$result				= true;
		//	
		
		$this->setLangCode($lang);
	
		// environment variables
		if ( false == putenv("LANGUAGE=" . $this->langCode ) )
		{
		 	Profiler::error('LANGUAGE variable cannot be set to ' . $this->langCode);
			$result = false;
		}		
		if ( false == putenv("LANG=" . $this->langCode ) )
		{
		 	Profiler::error('LANG variable cannot be set to ' . $this->langCode);
			$result = false;
		}
		if ( false == putenv("LC_ALL=" . $this->langCode ) )
		{
		 	Profiler::error('LC_ALL variable cannot be set to ' . $this->langCode);
			$result = false;
		}
		
		// locale for php application
		$localeSet = setlocale(LC_ALL, $this->langCode, $this->langCode . '' . strtolower($charset), $this->langCode . '.' . strtoupper($charset));
				
		if ( $localeSet != $this->langCode || empty($localeSet) )
		{
		 	Profiler::error(sprintf("Tried: setlocale to '%s', but could only set to '%s'.", $this->langCode, $localeSet) );
			$result = false;
		}
		
		// binding text locale to this application's domain
		$bindtextdomainSet = bindtextdomain($domain, $path );
		
		if ( empty($bindtextdomainSet) )
		{
		 	Profiler::error(sprintf("Tried: bindtextdomain, '%s', to directory, '%s', but received '%s'", $domain, $path, $bindtextdomainSet) );
			$result = false;
		}
		
		bind_textdomain_codeset($domain, $charset);
		
		$textdomainSet = textdomain($domain);
		if ( empty($textdomainSet) )
		{
		 	Profiler::error(sprintf("Tried: set textdomain to '%s', but got '%s'",$domain, $textdomainSet) );
			$result = false;
		}
		
		// and return result
		$this->gettext = $result;
		
		return $result;
    }
	
	/**
	 * Setter for the langCode property
	 * 
	 * @param string $lang
	 * 
	 * @access public 
	 */
	public function setLangCode($lang = '')
	{
		if (in_array($lang,$this->registry->supported_langs))
		{
			$this->langCode = $lang;
		}
	}
	
	/**
	 * Receives a text to be translated through gettext functions.
	 *
 	 * @param 	string  $string				string to be translated.
	 * @param	mixed	$params				array of parameters
	 * @param 	int		$params['count']	number of elements for plural forms
	 * @param 	string  $params['caps']		capitalize mode
	 *
	 * @return string	translated string, or original if no translation found
	 *
	 */
	function translate($string = '', $params = array())
	{
		// vars
		$count = 1;
		
		// plural or singular?
		if (isset($params['count']))
		{
			if(abs($params['count'] != 1 ))
			{
				$count = abs($params['count']);
			}
		}
		
		if ($this->gettext)
		{
			$string = ngettext($string, $string, $count);
		}
		else
		{
			$string = $this->getFromFile($string, $count);
		}
		
		// capitalization
		if (isset($params['caps']))
		{
			switch ($params['caps'])
			{
				case 'first':
					$string = ucfirst($string);
					break;
				case 'words':
					$string = ucwords($string);
					break;
				case 'upper':
					$string = strtoupper($string);
					break;
				case 'none':
				default:
			}	
		}
		
		// apply vars?
		if (isset($params['vars']))
		{
			if (is_array($params['vars']))
			{
				$params['vars'] = array_merge(array($count),$params['vars']);
			}
			else
			{
				$params['vars'] = array($count,$params['vars']);
			}
		}
		else
		{
			$params['vars'] = array($count);
		}
		
		$string = vsprintf($string,$params['vars']);
				
		// and it's done!
		return $string;
	}
	
	/**
	 * Retrieves a translated string from a plain php file
	 * 
	 * @param string to translate
	 * @param int $count for plural forms
	 * 
	 * @return translated string
	 * 
	 * @access private
	 */
	private function getFromFile($string = '', $count = 1)
	{
		if (!$this->langBuffer)
		{
			$this->langBuffer = $this->readLangFile();
		}
		
		if (isset($this->langBuffer[$string]))
		{
			if (is_array($this->langBuffer[$string])) 
			{
				if ( isset($this->langBuffer[$string][$count])) 
				{
					return $this->langBuffer[$string][$count];
				}
				else if (isset($this->langBuffer[$string]['n']))
				{
					return $this->langBuffer[$string]['n'];	
				}
			}
			else if (is_string($this->langBuffer[$string]))
			{
				return $this->langBuffer[$string];
			}
		}
		
		return $string;
	}	
	
	/**
	 * Reads the language file from disk
	 * 
	 * @return the $lang array
	 * 
	 * @access private
	 */
	private function readLangFile()
	{
		$file = __APP_PATH . '/lang/' . $this->langCode . '.php';
	
		if (file_exists($file))
		{
			include($file);
	
			return $lang;	
		}
		
		return null;
	}
}
?>